'use strict';
function buildPath(method, type, obj) {
    let path = '';
    let id = obj.db_id;

    switch (type) {
        case 'person':
            if (method == 'POST')
                path = 'persons'
            else
                path = 'person';
            break;
        case 'lead':
            if (method == 'POST')
                path = 'leads'
            else
                path = 'lead';
            break;
        case 'organization':
            if (method == 'POST')
                path = 'organizations'
            else
                path = 'organization';
            break;
        case 'property':
            if (method == 'POST')
                path = 'properties'
            else
                path = 'property';
            break;
        case 'property_type':
            path = 'property-types';
            break;
        case 'media_source':
            if (method == 'POST')
                path = 'media-sources'
            else
                path = 'media-source';
            break;
        case 'neighborhood':
            path = 'neighborhoods';
            break;
        case 'property_feature':
            if (method == 'POST')
                path = 'property-features'
            else
                path = 'property-feature';
            id = obj.db_id;
            break;
        case 'timeline':
            path = 'timeline'
            break;
        default:
            break;
    };
    if (method.match(/^(PUT|DELETE)$/i)) {
        path = path + '/' + id
    };

    return path;
}

function postObject(obj, apiUrl, objType, token, finish, current_database) {

  var xhr = new XMLHttpRequest();
  let method = '';
  let url = '';
  method = obj.action;
  url = apiUrl + '/' + buildPath(method, objType, obj);

  // Python server (API) does not support PUT method
  if (method == 'PUT') method = 'POST';
  xhr.onreadystatechange = function () {
    if (xhr.readyState === 4) {
        let response = (xhr.response == '')? {} : JSON.parse(xhr.response);
        if (xhr.status) {
            switch(xhr.status) {
                case 204:
                    obj.action = 'OK';
                    break;
                case 200:
                    obj.action = 'OK';
                    if (response.external_id)
                        obj.external_id = response.external_id;
                    if (response.code)
                        obj.code = response.code;
                    if (response.db_id)
                        obj.db_id = response.db_id;
                    break;
            };
            response.status = xhr.status;
        } else {
            response.status = 500;
        };
        response.method = method;
        response[objType] = obj;
        postMessage(response);

        if (finish == true) {
            sleep(5000);
            postMessage(JSON.parse(JSON.stringify({'sync': 'Finish', 'type': objType})));
        };
    };
  };

  xhr.open(method, url);
  xhr.setRequestHeader('Authorization', token);
  xhr.setRequestHeader('Content-Type', 'application/json');
  xhr.setRequestHeader('x-http-method-override', method);
  if (current_database) {
    xhr.setRequestHeader('X-Current-Database', current_database);
  }
  if (method == 'DELETE')
    xhr.send();
  else {
    let paramsjson = Object();
    if (objType == 'timeline')
      paramsjson[objType] = JSON.parse(JSON.stringify(obj.details))
    else
      paramsjson[objType] = JSON.parse(JSON.stringify(obj));
    xhr.send(JSON.stringify(paramsjson));
  }
};

function sleep(ms) {
	var start = new Date().getTime();
	for (var i = 0; i < 1e7; i++) {
		if ((new Date().getTime() - start) > ms) { break; }
	}
};

self.addEventListener('message', function(e) {

	let json = JSON.parse(e.data);
	let finish = false;
	let typeObject = '';
    let value = '';

	if (json['persons']) {
		typeObject = 'persons';
		value = 'person';
	}

    if (json['leads']) {
		typeObject = 'leads';
		value = 'lead';
	}

	if (json['organizations']) {
		typeObject = 'organizations';
		value = 'organization';
	}

	if (json['properties']) {
		typeObject = 'properties';
		value = 'property';
	}

	if (json['property_type']) {
		typeObject = 'property_type';
		value = 'property_type';
	}

	if (json['media_source']) {
		typeObject = 'media_source';
		value = 'media_source';
	}

	if (json['neighborhoods']) {
		typeObject = 'neighborhoods';
		value = 'neighborhood';
	}

	if (json['property_feature']) {
		typeObject = 'property_feature';
		value = 'property_feature';
    }

    if (json['timeline']) {
		typeObject = 'timeline';
		value = 'timeline';
    }

	for (var i = 0; i < json[typeObject].length; i++) {
		if (i == (json[typeObject].length - 1)) {
			finish = true;
		}
		if (json[typeObject][i].action != '' && json[typeObject][i].action != null) {
            let current_database = '';
            if (json['current_database']) {
                current_database = json['current_database'];
            }
			postObject(json[typeObject][i], json['apiUrl'], value, json['token'], finish, current_database);
        }
		sleep(200);
	};

}, false);